<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:ctx="http://www.ibm.com/2010/04/CompilerTransformation">

   <xsl:param name="LANG" select="ctx:XLTransformationReport/ctx:CompilationStep/ctx:StepDetails/ctx:Detail[ctx:FieldTitle = 'Locale']/ctx:FieldValue"/>
   <xsl:param name="LOCALE">
   <xsl:choose>
         <xsl:when test="$LANG = 'ja_JP'">Japanese.utf8</xsl:when>
         <xsl:when test="$LANG = 'ja_JP.eucjp'">Japanese.utf8</xsl:when>
         <xsl:when test="$LANG = 'ja_JP.utf8'">Japanese.utf8</xsl:when>
         <xsl:when test="$LANG = 'JA_JP'">Japanese.utf8</xsl:when>
         <xsl:when test="$LANG = 'Ja_JP'">Japanese.utf8</xsl:when>
         <xsl:when test="$LANG = 'zh_CN'">Chinese.utf8</xsl:when>
         <xsl:when test="$LANG = 'zh_CN.gb18030'">Chinese.utf8</xsl:when>
         <xsl:when test="$LANG = 'zh_CN.gb2312'">Chinese.utf8</xsl:when>
         <xsl:when test="$LANG = 'zh_CN.gbk'">Chinese.utf8</xsl:when>
         <xsl:when test="$LANG = 'zh_CN.utf8'">Chinese.utf8</xsl:when>
         <xsl:when test="$LANG = 'ZH_CN'">Chinese.utf8</xsl:when>
         <xsl:when test="$LANG = 'Zh_CN'">Chinese.utf8</xsl:when>
         <xsl:otherwise>English.utf8</xsl:otherwise>
   </xsl:choose>
   </xsl:param>
   <xsl:param name="xlstyle_msg" select="document(concat('./XMLMessages-',$LOCALE,'.xml'))/XMLMessages/xlstyle"/>
   <xsl:param name="xllisting_msg" select="document(concat('./XMLMessages-',$LOCALE,'.xml'))/XMLMessages/xllisting"/>
   <xsl:param name="XMLContent_msg" select="document(concat('./XMLMessages-',$LOCALE,'.xml'))/XMLMessages/XMLContent"/>

   <xsl:output method="html" encoding="UTF-8" indent="yes" doctype-public="-//W3C//DTD XHTML 1.0 Transitional//EN/" doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"/>
   
   <xsl:template match="/">
   
   <html xmlns="http://www.w3.org/1999/xhtml"> 
   <xsl:choose>
      <xsl:when test="$LOCALE = 'ja_JP'">
         <xsl:attribute name="lang">ja</xsl:attribute>
         <xsl:attribute name="dir">ltr</xsl:attribute>
      </xsl:when>
      <xsl:when test="$LOCALE = 'zh_CN'">
         <xsl:attribute name="lang">zh</xsl:attribute>
         <xsl:attribute name="dir">ltr</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
         <xsl:attribute name="lang">en</xsl:attribute>
         <xsl:attribute name="dir">ltr</xsl:attribute>
      </xsl:otherwise> 
   </xsl:choose>

      <head>
         <title>
            <xsl:call-template name="xlstyle"><xsl:with-param name="key">HTMLTitle</xsl:with-param></xsl:call-template>
            - <xsl:call-template name="xlstyle"><xsl:with-param name="key">Version</xsl:with-param></xsl:call-template><xsl:value-of select="ctx:XLTransformationReport/@version"/>
         </title>
         <style type="text/css">
              BODY
              {
                font-family: Arial;
                font-size: 12px;
              }
              TABLE.SIMPLE
              {
                font-family: Verdana, Geneva, Arial, Helvetica, sans-serif ;
                font-size: 11px;
                font-weight: normal;
                border: 0px;
                border-collapse: collapse;
                border-spacing: 0px;
              }
              TABLE.SIMPLE TD
              {
                border: 1px #C0C0C0 solid;
                padding: 4px;
              }
              TABLE.SIMPLE TD.HEADER
              {
                font-size: 12px;
                font-weight: bold;
                background-color: #E0E0E0;
                padding: 4px;
              }
         </style>
      </head>
      <body>
        <h2>
           <xsl:call-template name="xlstyle"><xsl:with-param name="key">DocHeader</xsl:with-param></xsl:call-template>
           - <xsl:call-template name="xlstyle"><xsl:with-param name="key">Version</xsl:with-param></xsl:call-template><xsl:value-of select="ctx:XLTransformationReport/@version"/>
        </h2>
        <xsl:apply-templates select="ctx:XLTransformationReport/ctx:CompilationStep/ctx:StepDetails/ctx:Detail" mode="show-details"/> 
      
        <h3><xsl:call-template name="xlstyle"><xsl:with-param name="key">TOC</xsl:with-param></xsl:call-template></h3>
        <ol>
          <li><a href="#program"><xsl:call-template name="xlstyle"><xsl:with-param name="key">ProgramHierarchy</xsl:with-param></xsl:call-template></a></li>
          <li><a href="#transformation"><xsl:call-template name="xlstyle"><xsl:with-param name="key">TransformationHierarchy</xsl:with-param></xsl:call-template></a></li>
          <li><a href="#profiling"><xsl:call-template name="xlstyle"><xsl:with-param name="key">ProfilingReports</xsl:with-param></xsl:call-template></a></li>
        </ol>

        <h3><a name="program"><xsl:call-template name="xlstyle"><xsl:with-param name="key">ProgramHierarchy</xsl:with-param></xsl:call-template></a></h3>
        <xsl:apply-templates select="ctx:XLTransformationReport/ctx:CompilationStep/ctx:ProgramHierarchy"/>
        <xsl:apply-templates select="ctx:XLTransformationReport/ctx:CompilationStep/ctx:ProgramHierarchy" mode="list-regions"/>
        <h3><a name="transformation"><xsl:call-template name="xlstyle"><xsl:with-param name="key">TransformationHierarchy</xsl:with-param></xsl:call-template></a></h3>
        <xsl:apply-templates select="ctx:XLTransformationReport/ctx:CompilationStep/ctx:TransformationHierarchy"/>
        <xsl:apply-templates select="ctx:XLTransformationReport/ctx:CompilationStep/ctx:TransformationHierarchy" mode="list-transforms"/>
        <h3><a name="profiling"><xsl:call-template name="xlstyle"><xsl:with-param name="key">ProfilingReports</xsl:with-param></xsl:call-template></a></h3>
        <xsl:apply-templates select="ctx:XLTransformationReport/ctx:CompilationStep/ctx:ProfilingReports"/>
        <xsl:apply-templates select="ctx:XLTransformationReport/ctx:CompilationStep/ctx:ProfilingReports" mode="list-profiling"/>
      </body>
   </html>
   </xsl:template>

   <xsl:template match="ctx:Detail" mode="show-details">
      <p><b><xsl:value-of select="ctx:FieldTitle"/></b>: <xsl:value-of select="ctx:FieldValue"/></p>   
   </xsl:template>

   <xsl:template match="ctx:ProgramHierarchy">
      <ul> 
         <xsl:apply-templates select="ctx:FileList/ctx:File"/>  
      </ul>
   </xsl:template>
   <xsl:template match="ctx:File">
      <li>
         <a name="file_{@id}">
         <b><xsl:call-template name="xlstyle"><xsl:with-param name="key">File</xsl:with-param></xsl:call-template> #<xsl:value-of select="@id"/></b>:
         <xsl:value-of select="@name"/> 
         </a>
         <ul>
            <xsl:apply-templates select="ctx:RegionList/ctx:Region"/>
         </ul>
      </li>
   </xsl:template>
   <xsl:template match="ctx:Region">
      <li>
         <a href="#reg_{@id}">
         <b><xsl:call-template name="xlstyle"><xsl:with-param name="key">Region</xsl:with-param></xsl:call-template> #<xsl:value-of select="@id"/></b>:
         <xsl:value-of select="@name"/>
         </a>
      </li>
   </xsl:template>

   <xsl:template match="ctx:ProgramHierarchy" mode="list-regions">
      <xsl:apply-templates select="ctx:FileList/ctx:File/ctx:RegionList/ctx:Region" mode="list-regions"/>
   </xsl:template>
   <xsl:template match="ctx:Region" mode="list-regions">
      <table width="100%">
         <tr>
         <td width="30%" valign="top">
            <table class="SIMPLE" width="100%">
               <tr><td class="HEADER" width="50%"><a name="reg_{@id}"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Region</xsl:with-param></xsl:call-template> #</a></td><td><xsl:value-of select="@id"/></td></tr>
               <tr><td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">RegionName</xsl:with-param></xsl:call-template></td><td><xsl:value-of select="@name"/></td></tr>
               <tr><td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">DRegionName</xsl:with-param></xsl:call-template></td>
               <td>
               <xsl:choose>
                  <xsl:when test="count(@demangledName) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
                  <xsl:otherwise><xsl:value-of select="@demangledName"/></xsl:otherwise>
               </xsl:choose>      
               </td>
               </tr>
               <tr><td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">StartLine</xsl:with-param></xsl:call-template> #</td>
               <td>
               <xsl:choose>
                  <xsl:when test="count(@startLineNumber) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
                  <xsl:otherwise><xsl:value-of select="@startLineNumber"/></xsl:otherwise>
               </xsl:choose>
               </td>
               </tr>
               <tr><td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">EndLine</xsl:with-param></xsl:call-template> #</td>
               <td>
               <xsl:choose>
                  <xsl:when test="count(@endLineNumber) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
                  <xsl:otherwise><xsl:value-of select="@endLineNumber"/></xsl:otherwise>
               </xsl:choose>
               </td>
               </tr>
            </table> 
         </td> 
         <td width="70%" valign="top">
            <table class="SIMPLE" width="100%">
               <tr><td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Pseudocode</xsl:with-param></xsl:call-template></td></tr>
               <tr>
               <td>
               <xsl:choose>
                  <xsl:when test="count(ctx:Pseudocode) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
                  <xsl:otherwise><xsl:apply-templates select="ctx:Pseudocode" mode="pseudocode-table"/></xsl:otherwise>
               </xsl:choose> 
               </td>
               </tr>
            </table> 
         </td>
         </tr>
         <tr>
            <table class="SIMPLE" width="100%">
               <tr><td class="HEADER" colspan="9"><xsl:call-template name="xlstyle"><xsl:with-param name="key">LoopTable</xsl:with-param></xsl:call-template></td></tr>
               <xsl:if test="count(ctx:LoopList/ctx:Loop) = 0">
                  <tr><td><xsl:call-template name="xlstyle"><xsl:with-param name="key">NoLoopInfo</xsl:with-param></xsl:call-template></td></tr>
               </xsl:if>
               <xsl:if test="count(ctx:LoopList/ctx:Loop) > 0">
                  <tr>
                  <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">LoopIndex</xsl:with-param></xsl:call-template></td>
                  <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">StartLine</xsl:with-param></xsl:call-template> #</td>
                  <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">EndLine</xsl:with-param></xsl:call-template> #</td>
                  <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">PLoopIndex</xsl:with-param></xsl:call-template></td>
                  <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NestLevel</xsl:with-param></xsl:call-template></td>
                  <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">MinCost</xsl:with-param></xsl:call-template></td>
                  <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">MaxCost</xsl:with-param></xsl:call-template></td>
                  <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">IterCount</xsl:with-param></xsl:call-template></td>
                  <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Attributes</xsl:with-param></xsl:call-template></td>
                  </tr>
                  <xsl:apply-templates select="ctx:LoopList/ctx:Loop" mode="list-loops"/>
               </xsl:if> 
            </table>
         </tr>
      </table>   
      <br/>
   </xsl:template>
   <xsl:template match="ctx:Pseudocode" mode="pseudocode-table">
      <pre><xsl:apply-templates select="*" mode="pseudocode-table"/></pre>
   </xsl:template>
   <xsl:template match="ctx:PseudoLoop" mode="pseudocode-table">
      <xsl:apply-templates select="*" mode="pseudocode-table"/>
   </xsl:template>
   <xsl:template match="ctx:PseudoBlock" mode="pseudocode-table">
      <xsl:apply-templates select="*" mode="pseudocode-table"/>
   </xsl:template>
   <xsl:template match="ctx:PseudoLine" mode="pseudocode-table">
      <xsl:value-of select="."/>
   </xsl:template>
   <xsl:template match="ctx:Loop" mode="list-loops">
      <tr>
      <td><xsl:value-of select="@index"/></td>
      <td>
         <xsl:choose>
            <xsl:when test="count(@startLineNumber) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
            <xsl:otherwise><xsl:value-of select="@startLineNumber"/></xsl:otherwise>
         </xsl:choose>
      </td>
      <td>
         <xsl:choose>
            <xsl:when test="count(@endLineNumber) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
            <xsl:otherwise><xsl:value-of select="@endLineNumber"/></xsl:otherwise>
         </xsl:choose>
      </td>
      <td>
         <xsl:choose>
            <xsl:when test="count(@parentLoopIndex) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
            <xsl:otherwise><xsl:value-of select="@parentLoopIndex"/></xsl:otherwise>
         </xsl:choose>
      </td>
      <td>
         <xsl:choose>
            <xsl:when test="count(@nestLevel) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
            <xsl:otherwise><xsl:value-of select="@nestLevel"/></xsl:otherwise>
         </xsl:choose>
      </td>
      <td>
         <xsl:choose>
            <xsl:when test="count(@minCost) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
            <xsl:otherwise><xsl:value-of select="@minCost"/></xsl:otherwise>
         </xsl:choose>
      </td>
      <td>
         <xsl:choose>
            <xsl:when test="count(@maxCost) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
            <xsl:otherwise><xsl:value-of select="@maxCost"/></xsl:otherwise>
         </xsl:choose>
      </td>
      <td>
         <xsl:choose>
            <xsl:when test="count(@iterCount) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
            <xsl:otherwise><xsl:value-of select="@iterCount"/> (<xsl:value-of select="@iterKind"/>)</xsl:otherwise>
         </xsl:choose>
      </td>
      <td>
         <ul>
            <xsl:apply-templates select="@residual"/>
            <xsl:apply-templates select="@perfectNest"/>
            <xsl:apply-templates select="@wellBehaved"/>
            <xsl:apply-templates select="@mayThrow"/>
            <xsl:apply-templates select="@bumpNormalized"/>
            <xsl:apply-templates select="@guarded"/>
            <xsl:apply-templates select="@irreducible"/>
            <xsl:apply-templates select="@lowerBoundNormalized"/>
         </ul>
      </td>
      </tr>
   </xsl:template>
   <xsl:template match="@residual[. = 'true']">
      <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">Residual</xsl:with-param></xsl:call-template></li>
   </xsl:template>
   <xsl:template match="@residual[. = 'false']"/>
   <xsl:template match="@perfectNest[. = 'true']">
      <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">PerfectNest</xsl:with-param></xsl:call-template></li>
   </xsl:template>
   <xsl:template match="@perfectNest[. = 'false']"/>
   <xsl:template match="@wellBehaved[. = 'true']">
      <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">WellBehaved</xsl:with-param></xsl:call-template></li>
   </xsl:template>
   <xsl:template match="@wellBehaved[. = 'false']"/>
   <xsl:template match="@mayThrow[. = 'true']">
      <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">MayThrow</xsl:with-param></xsl:call-template></li>
   </xsl:template>
   <xsl:template match="@mayThrow[. = 'false']"/>
   <xsl:template match="@bumpNormalized[. = 'true']">
      <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">BumpNormal</xsl:with-param></xsl:call-template></li>
   </xsl:template>
   <xsl:template match="@bumpNormalized[. = 'false']"/>
   <xsl:template match="@guarded[. = 'true']">
      <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">Guarded</xsl:with-param></xsl:call-template></li>
   </xsl:template>
   <xsl:template match="@guarded[. = 'false']"/>
   <xsl:template match="@irreducible[. = 'true']">
      <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">Irreducible</xsl:with-param></xsl:call-template></li>
   </xsl:template>
   <xsl:template match="@irreducible[. = 'false']"/>
   <xsl:template match="@lowerBoundNormalized[. = 'true']">
      <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">LowerBound</xsl:with-param></xsl:call-template></li>
   </xsl:template>
   <xsl:template match="@lowerBoundNormalized[. = 'false']"/>

   <xsl:template match="ctx:TransformationHierarchy">
      <ul>
         <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">IntraTransform</xsl:with-param></xsl:call-template></li> 
         <xsl:choose>
            <xsl:when test="count(ctx:IntraTransformationList) = 0">
               <ul><li><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></li></ul>
            </xsl:when>
            <xsl:otherwise><xsl:apply-templates select="ctx:IntraTransformationList"/></xsl:otherwise>
         </xsl:choose>
         <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">InterTransform</xsl:with-param></xsl:call-template></li>
         <xsl:choose>
            <xsl:when test="count(ctx:InterTransformationList) = 0">
               <ul><li><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></li></ul>
            </xsl:when>
            <xsl:otherwise><xsl:apply-templates select="ctx:InterTransformationList"/></xsl:otherwise>
         </xsl:choose>
      </ul>
   </xsl:template>
   <xsl:template match="ctx:IntraTransformationList">
      <ul>
         <xsl:apply-templates select="ctx:LoopTransformationList"/>
      </ul>
   </xsl:template>
   <xsl:template match="ctx:InterTransformationList">
      <ul>
         <xsl:apply-templates select="ctx:InlineOptimizationList"/>
         <xsl:apply-templates select="ctx:DataReorganizationList"/>
      </ul>
   </xsl:template>
   <xsl:template match="ctx:LoopTransformationList">
      <li><a href="#loopTransformation"><xsl:call-template name="xlstyle"><xsl:with-param name="key">LoopTransform</xsl:with-param></xsl:call-template></a></li>
      <ul>
         <xsl:apply-templates select="ctx:LoopTransform"/>
      </ul>
   </xsl:template>
   <xsl:template match="ctx:InlineOptimizationList">
      <li><a href="#inlineOptimization"><xsl:call-template name="xlstyle"><xsl:with-param name="key">InlineOpt</xsl:with-param></xsl:call-template></a></li>
      <ul>
         <xsl:apply-templates select="ctx:InlineOpt"/>
      </ul>
   </xsl:template>
   <xsl:template match="ctx:DataReorganizationList">
      <li><a href="#dataReorganization"><xsl:call-template name="xlstyle"><xsl:with-param name="key">DataReorg</xsl:with-param></xsl:call-template></a></li>
      <ul>
         <xsl:apply-templates select="ctx:DataReorg"/>
      </ul>
   </xsl:template>
   <xsl:template match="ctx:LoopTransform">
      <li><a href="#loopTrans_{@seq}"><b><xsl:call-template name="xlstyle"><xsl:with-param name="key">Seq</xsl:with-param></xsl:call-template> #<xsl:value-of select="@seq"/></b>: <xsl:value-of select="@type"/></a></li>
   </xsl:template>
   <xsl:template match="ctx:InlineOpt">
      <li><a href="#inlineOpt_{@seq}"><b><xsl:call-template name="xlstyle"><xsl:with-param name="key">Seq</xsl:with-param></xsl:call-template> #<xsl:value-of select="@seq"/></b>: <xsl:value-of select="@type"/></a></li>
   </xsl:template>
   <xsl:template match="ctx:DataReorg">
      <li><a href="#dataReorg_{@seq}"><b><xsl:call-template name="xlstyle"><xsl:with-param name="key">Seq</xsl:with-param></xsl:call-template> #<xsl:value-of select="@seq"/></b>: <xsl:value-of select="@type"/></a></li>
   </xsl:template>
   
   <xsl:template match="ctx:TransformationHierarchy" mode="list-transforms">
      <xsl:apply-templates select="ctx:IntraTransformationList/ctx:LoopTransformationList" mode="list-transforms"/>
      <xsl:apply-templates select="ctx:InterTransformationList/ctx:InlineOptimizationList" mode="list-transforms"/>
      <xsl:apply-templates select="ctx:InterTransformationList/ctx:DataReorganizationList" mode="list-transforms"/>
   </xsl:template>

   <xsl:template match="ctx:LoopTransformationList" mode="list-transforms">
      <h4><a name="loopTransformation"><xsl:call-template name="xlstyle"><xsl:with-param name="key">LoopTransform</xsl:with-param></xsl:call-template></a></h4>
      <table class="SIMPLE" width="100%">
         <tr>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Seq</xsl:with-param></xsl:call-template> #</td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Type</xsl:with-param></xsl:call-template></td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Phase</xsl:with-param></xsl:call-template></td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Region</xsl:with-param></xsl:call-template> #</td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Line</xsl:with-param></xsl:call-template> #</td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">LoopIndex</xsl:with-param></xsl:call-template></td>
            <td class="HEADER"><xsl:call-template name="XMLContent"><xsl:with-param name="key">Description</xsl:with-param></xsl:call-template></td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Attributes</xsl:with-param></xsl:call-template></td>
         </tr>
         <xsl:apply-templates select="ctx:LoopTransform" mode="list-transforms"/>
      </table>
   </xsl:template>
   <xsl:template match="ctx:LoopTransform" mode="list-transforms">
      <tr>
         <td><a name="loopTrans_{@seq}"><xsl:value-of select="@seq"/></a></td>
         <td><xsl:value-of select="@type"/> (<xsl:value-of select="@result"/>)</td>
         <td><xsl:value-of select="@phase"/></td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@regionId) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><a href="#reg_{@regionId}"><xsl:value-of select="@regionId"/></a></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@lineNumber) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@lineNumber"/></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@loopIndex) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@loopIndex"/></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:call-template name="xllisting"><xsl:with-param name="key"><xsl:value-of select="@type"/></xsl:with-param></xsl:call-template>
         </td> 
         <td>
            <xsl:choose>
               <xsl:when test="count(ctx:AttributeList) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:apply-templates select="ctx:AttributeList" mode="list-transforms"/></xsl:otherwise> 
            </xsl:choose>
         </td>
      </tr>
   </xsl:template>
   <xsl:template match="ctx:AttributeList" mode="list-transforms">
      <ul>
         <xsl:apply-templates select="ctx:Attribute" mode="list-transforms"/>
      </ul>
   </xsl:template>
   <xsl:template match="ctx:Attribute" mode="list-transforms">
      <li><xsl:value-of select="ctx:AttributeTitle"/>: <xsl:value-of select="ctx:AttributeValue"/></li>
   </xsl:template>
  
   <xsl:template match="ctx:InlineOptimizationList" mode="list-transforms">
      <h4><a name="inlineOptimization"><xsl:call-template name="xlstyle"><xsl:with-param name="key">InlineOpt</xsl:with-param></xsl:call-template></a></h4>
      <table class="SIMPLE" width="100%">
         <tr>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Seq</xsl:with-param></xsl:call-template> #</td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Type</xsl:with-param></xsl:call-template></td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Phase</xsl:with-param></xsl:call-template></td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CallerRegion</xsl:with-param></xsl:call-template> #</td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CalleeRegion</xsl:with-param></xsl:call-template> #</td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CallsiteFile</xsl:with-param></xsl:call-template> #</td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CallsiteLine</xsl:with-param></xsl:call-template> #</td>
            <td class="HEADER"><xsl:call-template name="XMLContent"><xsl:with-param name="key">Description</xsl:with-param></xsl:call-template></td>
         </tr>
         <xsl:apply-templates select="ctx:InlineOpt" mode="list-transforms"/>
      </table>
   </xsl:template>
   <xsl:template match="ctx:InlineOpt" mode="list-transforms">
      <tr>
         <td><a name="inlineOpt_{@seq}"><xsl:value-of select="@seq"/></a></td>
         <td><xsl:value-of select="@type"/> (<xsl:value-of select="@result"/>)</td>
         <td><xsl:value-of select="@phase"/></td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@callerRegionId) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><a href="#reg_{@callerRegionId}"><xsl:value-of select="@callerRegionId"/></a></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@calleeRegionId) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><a href="#reg_{@calleeRegionId}"><xsl:value-of select="@calleeRegionId"/></a></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@callsiteFileId) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><a href="#file_{@callsiteFileId}"><xsl:value-of select="@callsiteFileId"/></a></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@callsiteLineNumber) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@callsiteLineNumber"/></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:call-template name="xllisting"><xsl:with-param name="key"><xsl:value-of select="@type"/></xsl:with-param></xsl:call-template>
         </td> 
      </tr>
   </xsl:template>

   <xsl:template match="ctx:DataReorganizationList" mode="list-transforms">
      <h4><a name="dataReorganization"><xsl:call-template name="xlstyle"><xsl:with-param name="key">DataReorg</xsl:with-param></xsl:call-template></a></h4>
      <table class="SIMPLE" width="100%">
         <tr>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Seq</xsl:with-param></xsl:call-template> #</td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Type</xsl:with-param></xsl:call-template></td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Phase</xsl:with-param></xsl:call-template></td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">DataName</xsl:with-param></xsl:call-template></td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Category</xsl:with-param></xsl:call-template></td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Region</xsl:with-param></xsl:call-template> #</td>
            <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Line</xsl:with-param></xsl:call-template> #</td>
            <td class="HEADER"><xsl:call-template name="XMLContent"><xsl:with-param name="key">Description</xsl:with-param></xsl:call-template></td>
         </tr>
         <xsl:apply-templates select="ctx:DataReorg" mode="list-transforms"/>
      </table>
   </xsl:template>
   <xsl:template match="ctx:DataReorg" mode="list-transforms">
      <tr>
         <td><a name="dataReorg_{@seq}"><xsl:value-of select="@seq"/></a></td>
         <td><xsl:value-of select="@type"/></td>
         <td><xsl:value-of select="@phase"/></td>
         <td><xsl:value-of select="@dataName"/></td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@category) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@category"/></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@regionId) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@regionId"/></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@lineNumber) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@lineNumber"/></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:call-template name="xllisting"><xsl:with-param name="key"><xsl:value-of select="@type"/></xsl:with-param></xsl:call-template>
         </td>
      </tr>
   </xsl:template>
 
   <xsl:template match="ctx:ProfilingReports">
      <xsl:choose>
         <xsl:when test="count(ctx:BlockCounterList) = 0 and count(ctx:CallCounterList) = 0 and count(ctx:CacheMissList) = 0">
            <xsl:call-template name="xlstyle"><xsl:with-param name="key">NoProfInfo</xsl:with-param></xsl:call-template>
         </xsl:when>
         <xsl:otherwise>
            <ul>
               <xsl:apply-templates select="ctx:BlockCounterList"/>
               <xsl:apply-templates select="ctx:CallCounterList"/>
               <xsl:apply-templates select="ctx:CacheMissList"/>
            </ul> 
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>
   <xsl:template match="ctx:BlockCounterList">
      <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">BlockCounter</xsl:with-param></xsl:call-template></li>
      <ul>
         <xsl:apply-templates select="ctx:BlockCounter"/>
      </ul>
   </xsl:template>
   <xsl:template match="ctx:CallCounterList">
      <li><xsl:call-template name="xlstyle"><xsl:with-param name="key">CallCounter</xsl:with-param></xsl:call-template></li>
      <ul>
         <xsl:apply-templates select="ctx:CallCounter"/>
      </ul>
   </xsl:template>
   <xsl:template match="ctx:CacheMissList">
      <li><a href="#cacheMiss"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CacheMiss</xsl:with-param></xsl:call-template></a></li>
   </xsl:template>
   <xsl:template match="ctx:BlockCounter">
      <li>
      <a href="#bc_{@regionId}">
      <xsl:call-template name="xlstyle"><xsl:with-param name="key">Region</xsl:with-param></xsl:call-template>
      #<xsl:value-of select="@regionId"/>
      </a>
      </li> 
   </xsl:template>
   <xsl:template match="ctx:CallCounter">
      <li>
      <a href="#cc_{@regionId}">
      <xsl:call-template name="xlstyle"><xsl:with-param name="key">Region</xsl:with-param></xsl:call-template>
      #<xsl:value-of select="@regionId"/>
      </a>
      </li> 
   </xsl:template>

   <xsl:template match="ctx:ProfilingReports" mode="list-profiling">
      <xsl:apply-templates select="ctx:BlockCounterList" mode="list-profiling"/>
      <xsl:apply-templates select="ctx:CallCounterList" mode="list-profiling"/>
      <xsl:apply-templates select="ctx:CacheMissList" mode="list-profiling"/>
   </xsl:template>
   <xsl:template match="ctx:BlockCounterList" mode="list-profiling">
      <h4><a name="cacheMiss"><xsl:call-template name="xlstyle"><xsl:with-param name="key">BlockCounter</xsl:with-param></xsl:call-template></a></h4>
      <xsl:apply-templates select="ctx:BlockCounter" mode="list-profiling"/>
   </xsl:template>
   <xsl:template match="ctx:CallCounterList" mode="list-profiling">
      <h4><a name="cacheMiss"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CallCounter</xsl:with-param></xsl:call-template></a></h4>
      <xsl:apply-templates select="ctx:CallCounter" mode="list-profiling"/>
   </xsl:template>
   <xsl:template match="ctx:CacheMissList" mode="list-profiling">
      <h4><a name="cacheMiss"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CacheMiss</xsl:with-param></xsl:call-template></a></h4>
      <table class="SIMPLE">
         <tr>
         <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Memory</xsl:with-param></xsl:call-template></td>
         <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Region</xsl:with-param></xsl:call-template> #</td>
         <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Line</xsl:with-param></xsl:call-template> #</td>
         <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CacheLevel</xsl:with-param></xsl:call-template></td>
         <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">MissCount</xsl:with-param></xsl:call-template></td>
         <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">MissRate</xsl:with-param></xsl:call-template></td>
         </tr>
         <xsl:apply-templates select="ctx:CacheMiss" mode="list-profiling"/>
      </table> 
   </xsl:template>
   <xsl:template match="ctx:BlockCounter" mode="list-profiling">
      <table width="100%">
         <tr>
         <td width="30%" valign="top">
            <table class="SIMPLE" width="100%">
              <tr>
              <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Region</xsl:with-param></xsl:call-template> #</td>
              <td><a href="#reg_{@regionId}" name="bc_{@regionId}"><xsl:value-of select="@regionId"/></a></td>
              </tr>
              <tr>
              <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">RExecCount</xsl:with-param></xsl:call-template></td>
              <td><xsl:value-of select="@execCount"/></td>
              </tr>
              <tr>
              <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">BlockCoverage</xsl:with-param></xsl:call-template></td>
              <td>
                 <xsl:choose>
                    <xsl:when test="count(@coveredBlock) = 0 or count(@totalBlock) = 0">
                       <xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                       <xsl:value-of select="@coveredBlock"/> / <xsl:value-of select="@totalBlock"/>
                    </xsl:otherwise>
                 </xsl:choose>
              </td>
              </tr>
            </table>
         </td>
         <td width="70%" valign="top">
            <table class="SIMPLE" width="100%">
               <tr><td class="HEADER" colspan="4"><xsl:call-template name="xlstyle"><xsl:with-param name="key">BlockCounters</xsl:with-param></xsl:call-template></td></tr>
               <xsl:choose>
                  <xsl:when test="count(ctx:BlockList) = 0">
                     <tr><td><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></td></tr>
                  </xsl:when>
                  <xsl:otherwise>
                     <tr>
                     <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">BlockIndex</xsl:with-param></xsl:call-template></td>
                     <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">BExecCount</xsl:with-param></xsl:call-template></td>
                     <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">StartLine</xsl:with-param></xsl:call-template> #</td>
                     <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">EndLine</xsl:with-param></xsl:call-template> #</td>
                     </tr>
                     <xsl:apply-templates select="ctx:BlockList/ctx:Block" mode="list-profiling"/>
                  </xsl:otherwise>
               </xsl:choose>
            </table>
         </td>
         </tr>
      </table>
   </xsl:template>
   <xsl:template match="ctx:Block" mode="list-profiling">
      <tr>
         <td><xsl:value-of select="@index"/></td>
         <td><xsl:value-of select="@execCount"/></td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@startLineNumber) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@startLineNumber"/></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@endLineNumber) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@endLineNumber"/></xsl:otherwise>
            </xsl:choose>
         </td>
      </tr>
   </xsl:template>
   <xsl:template match="ctx:CallCounter" mode="list-profiling">
      <table width="100%">
         <tr>
         <td width="30%" valign="top">
            <table class="SIMPLE" width="100%">
              <tr>
              <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Region</xsl:with-param></xsl:call-template> #</td>
              <td><a href="#reg_{@regionId}" name="cc_{@regionId}"><xsl:value-of select="@regionId"/></a></td>
              </tr>
              <tr>
              <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">RExecCount</xsl:with-param></xsl:call-template></td>
              <td><xsl:value-of select="@execCount"/></td>
              </tr>
              <tr>
              <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CallCoverage</xsl:with-param></xsl:call-template></td>
              <td>
                 <xsl:choose>
                    <xsl:when test="count(@coveredCall) = 0 or count(@totalCall) = 0">
                       <xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                       <xsl:value-of select="@coveredCall"/> / <xsl:value-of select="@totalCall"/>
                    </xsl:otherwise>
                 </xsl:choose>
              </td>
              </tr>
            </table>
         </td>
         <td width="70%" valign="top">
            <table class="SIMPLE" width="100%">
               <tr><td class="HEADER" colspan="4"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CallCounters</xsl:with-param></xsl:call-template></td></tr>
               <xsl:choose>
                  <xsl:when test="count(ctx:CallList) = 0">
                     <tr><td><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></td></tr>
                  </xsl:when>
                  <xsl:otherwise>
                     <tr>
                     <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CallName</xsl:with-param></xsl:call-template></td>
                     <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">CExecCount</xsl:with-param></xsl:call-template></td>
                     <td class="HEADER"><xsl:call-template name="xlstyle"><xsl:with-param name="key">Line</xsl:with-param></xsl:call-template> #</td>
                     </tr>
                     <xsl:apply-templates select="ctx:CallList/ctx:Call" mode="list-profiling"/>
                  </xsl:otherwise>
               </xsl:choose>
            </table>
         </td>
         </tr>
      </table>
   </xsl:template>
   <xsl:template match="ctx:Call" mode="list-profiling">
      <tr>
         <td><xsl:value-of select="@name"/></td>
         <td><xsl:value-of select="@execCount"/></td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@lineNumber) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@lineNumber"/></xsl:otherwise>
            </xsl:choose>
         </td>
      </tr>
   </xsl:template>
   <xsl:template match="ctx:CacheMiss" mode="list-profiling">
      <tr>
         <td><xsl:value-of select="@memoryReference"/></td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@regionId) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><a href="#reg_{@regionId}"><xsl:value-of select="@regionId"/></a></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@lineNumber) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@lineNumber"/></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@cacheLevel) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@cacheLevel"/></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@cacheMissCount) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@cacheMissCount"/></xsl:otherwise>
            </xsl:choose>
         </td>
         <td>
            <xsl:choose>
               <xsl:when test="count(@cacheMissRate) = 0"><xsl:call-template name="xlstyle"><xsl:with-param name="key">NotAvailable</xsl:with-param></xsl:call-template></xsl:when>
               <xsl:otherwise><xsl:value-of select="@cacheMissRate"/></xsl:otherwise>
            </xsl:choose>
         </td>
      </tr>
   </xsl:template>

   <xsl:template name="xllisting">
      <xsl:param name="key"/>
      <xsl:choose>
         <xsl:when test="$xllisting_msg/message[@id = $key]">
            <xsl:value-of select="$xllisting_msg/message[@id = $key]"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="$key"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template name="xlstyle">
      <xsl:param name="key"/>
      <xsl:choose>
         <xsl:when test="$xlstyle_msg/message[@id = $key]">
            <xsl:value-of select="$xlstyle_msg/message[@id = $key]"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="$key"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template name="XMLContent">
      <xsl:param name="key"/>
      <xsl:choose>
         <xsl:when test="$XMLContent_msg/message[@id = $key]">
            <xsl:value-of select="$XMLContent_msg/message[@id = $key]"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="$key"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

</xsl:stylesheet>
